
import java.util.HashMap;

public class NLComposite extends NLTerm {
    
// ----------------------------------------------------------------------------
// Constructor
// ----------------------------------------------------------------------------

  public NLComposite(String s, NLTerm[] tt) {
  //If it is a 0-ary function, just pass a null pointer for the
  //NLTerm array.
    fstr = s; args = tt;
  }

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  String fstr;
  NLTerm[] args;
  String[] argStrings;
  boolean forceWords = false;
  boolean isInWords = false;
  String translation = "";
  
  static final int DEFAULT = 1;
  static final int WORDS = 2;

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
  
  public String xlat(int code) {
    String x = "";
    processArgs(code);
    String arity = getArity();
    HashMap map = NLLookup.lookUp(fstr,arity);
    String phrase = getPhrase(code,map);
        //diag:
        if (false) {
          System.out.println(
            (String) map.get("symb") + "\n" +
            (String) map.get("word")
          );
        }
        //
    if (phrase==null) {
      x = makeDefaultPhrase(code);
    } else {
      NLPhraseEvaluator ev = new NLPhraseEvaluator(phrase);
      x = ev.eval(argStrings);
    }
    this.translation = x;
    return x;
  }
  
  public boolean isInWords() {
  //Should call xlat before this method.
    return isInWords;
  }

  public String getString() {
  //Should call xlat before this method.
    return translation;
  }

  void processArgs(int code) {
    if (args!=null) {
      argStrings = new String[args.length];
      for (int i = 0; i < args.length; i++) {
        argStrings[i] = args[i].xlat(NLInfixrel.DEFAULT_CODE);
        forceWords |= args[i].isInWords();
      }
    }
  }
  
  String getArity() {
    String a = "0";
    if (args!=null) {
      a = Integer.toString(args.length);
    }
    return a;
  }
  
  String makeDefaultPhrase(int code) {
  //processArgs should already have been called by this time.
  //For use when the symbol could not be found in the dictionary.
  //We construct the default phrase, which is just the symbol followed
  //by its arguments (if any), in the appropriate sort of braces; and
  //possibly preceded by a negation symbol.
    String r = "$"+fstr;
    if (args!=null && args.length>0) {
      r += "($";
      for (int i = 0; i < argStrings.length; i++) {
        r += (i==0?"":",")+argStrings[i];
      }
      r += "$)$";
    } else {
      r += "$";
    }
    return r;
  }

  
  String getPhrase(int code, HashMap map) {
    String phrase = null;
    //forceWords should be set up by this time.
    if (forceWords) {
      code |= WORDS;
      code &= (~DEFAULT);
    }
    //Now proceed.
    //First determine if there is a symbolic phrase available.
    boolean symbolsAvailable = false;
    phrase = (String) map.get(GrammarConstants.FUNCSYMB);
    symbolsAvailable = (phrase!=null);
    //At this point phrase is already set up, provided we want
    //symbols.
    if ((code&WORDS)!=0 || !symbolsAvailable) {
    //We want words.
      phrase = (String) map.get(GrammarConstants.FUNCWORD);
      this.isInWords = true;
    }
    return phrase;
  }

}










