 

import java.util.HashMap;

public class NLQuant extends NLFormula {

// ----------------------------------------------------------------------------
// Fields
// ----------------------------------------------------------------------------

  boolean asserted;
  String type;
  NLFormula rho,phi,psi;
  NLInfixrel alpha;
  NLTerm[] tt,uu,vv;
  
  boolean isInWords = false;
  String translation = "";
  
  HashMap symbMap, wordMap;
  
  boolean diag = false;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public NLQuant(boolean asserted, String type, NLTerm[] tt, NLFormula rho,
                 NLFormula phi, NLInfixrel alpha, NLTerm[] uu, NLFormula psi,
                 NLTerm[] vv) {
    this.asserted = asserted; this.type = type; this.tt = tt;
    this.rho = rho; this.phi = phi; this.alpha = alpha;
    this.uu = uu; this.psi = psi; this.vv = vv;
    
    symbMap = new HashMap();
    symbMap.put("forall","$\\forall$");
    symbMap.put("exists","$\\exists$");
    symbMap.put("existsUnique","$\\exists !$");
    
    wordMap = new HashMap();
    wordMap.put("foralltrue","for every");
      String temp = (tt.length>1?"exist":"exists");
    wordMap.put("existstrue","there "+temp);
      temp = (tt.length>1?"exist":"exists a");
    wordMap.put("existsUniquetrue","there "+temp+" unique");
    wordMap.put("forallfalse","not for every");
      temp = (tt.length>1?"do":"does");
    wordMap.put("existsfalse","there "+temp+" not exist");
      temp = (tt.length>1?"are not":"is not a");
    wordMap.put("existsUniquefalse","there "+temp+" unique");
    
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  public String xlat(int mode) {
    //First call the xlat methods of the children, and figure out
    //if we are forced to return words.
    //Start with tt.
    for (NLTerm term : tt) {
      term.xlat(mode); this.isInWords |= term.isInWords();
//diag:
  if (diag) {
    System.out.printf("term is %s%nand this %s in words.%n",
      term.getString(),(term.isInWords()?"is":"is not")
    );
  }
//
    }
    //Treat the prepositional alpha, if it is there.
    if (alpha!=null) {
      boolean prepWords = this.isInWords();
      if (uu==null) {System.err.println("Prep. infix w/o RHS!");} else {
        for (int i = 0; i < uu.length; i++) {
          uu[i].xlat(mode); prepWords |= uu[i].isInWords();
//diag:
  if (diag) {
    System.out.printf("uu[%d] is %s%nand this %s in words.%n",
      i,uu[i].getString(),(uu[i].isInWords()?"is":"is not")
    );
  }
//
        }
      }
      if (prepWords || mode==NLPhrase.WORDS_MODE) {
        alpha.xlat(NLInfixrel.QUALIFIER_CODE);
      }
      else {alpha.xlat(NLInfixrel.DEFAULT_CODE);}
      prepWords |= alpha.isInWords();
//diag:
  if (diag && false) {
    System.out.printf("alpha %s in words.%n",
      (alpha.isInWords()?"is":"is not")
    );
  }
//
      this.isInWords |= prepWords;
    }
    //Rho, if it is there.
    if (rho!=null) {
      rho.xlat(mode); this.isInWords |= rho.isInWords();
//diag:
  if (diag) {
    System.out.printf("rho %s in words.%n",
      (rho.isInWords()?"is":"is not")
    );
  }
//
    }
    //Phi, if it is there.
    if (phi!=null) {
      phi.xlat(mode); this.isInWords |= phi.isInWords();
//diag:
  if (diag) {
    System.out.printf("phi %s in words.%n",
      (phi.isInWords()?"is":"is not")
    );
  }
//
    }
    //Psi, if it is there.
    if (psi!=null) {
      psi.xlat(mode); this.isInWords |= psi.isInWords();
//diag:
  if (diag) {
    System.out.printf("psi %s in words.%n",
      (psi.isInWords()?"is":"is not")
    );
  }
//
    }
    //vv, if there are there.
    if (vv!=null) {
      for (NLTerm term : vv) {
        term.xlat(mode); this.isInWords |= term.isInWords();
      }
    }
    //Finally, write the string.
    String x = "";
    //Let's always produce words, on quantified sentences.
    x = writeWords();
    this.isInWords = true;
/*
    if (this.isInWords || mode==NLPhrase.WORDS_MODE) {
      x = writeWords();
    } else {
      x = writeSymbols();
    }
*/
    this.translation = x;
//diag:
  if (diag) {
    System.out.println();
  }
//
    return x;
  }
  
  public boolean isInWords() {
  //Should be called only *after* xlat.
    return this.isInWords;
  }

  public String getString() {
  //Should be called only *after* xlat.
    return this.translation;
  }
   
  String writeSymbols() {
    String s = "";
      s += (asserted?"":"\\neg ");
      s += "("+symbMap.get(type)+" ";
      for (int i = 0; i < tt.length; i++) {
        s += (i>0?",":"")+tt[i].getString();
      }
      if (alpha!=null) {
        s += " "+alpha.getString()+" ";
        for (int i = 0; i < uu.length; i++) {
          s += (i>0?",":"")+uu[i].getString();
        }
        s += ")";
      }
      if (rho!=null) {
        s += " : "+rho.getString()+")"+phi.getString();
      } else {
        if (phi!=null) {
          s += phi.getString();
        } else {
          s += "("+psi.getString()+", ";
          for (int i = 0; i < vv.length; i++) {
            s += (i>0?",":"")+vv[i].getString();
          }
          s += " fixed";
        }
      }
    return s;
  }
  
  String writeWords() {    
    String s = "";
      String temp = type+Boolean.toString(asserted);
      s += wordMap.get(temp)+" ";
      for (int i = 0; i < tt.length; i++) {
        s += (i>0?", ":"")+tt[i].getString();
      }
      if (alpha!=null) {
        s += " "+alpha.getString()+" ";
        for (int i = 0; i < uu.length; i++) {
          s += (i>0?", ":"")+uu[i].getString();
        }
      }
      if (rho!=null) {
        s += " such that ";
        s += rho.getString();
        s += (type.equals("forall")?", we have that ":", and ");
        s += phi.getString();
      } else {
        s += (type.equals("forall")?", ":" such that ");
        if (phi!=null) {
          s += phi.getString();
        } else {
          s += psi.getString();
          //s += ", where ";
          //for (int i = 0; i < vv.length; i++) {
          //  s += (i>0?", ":"")+vv[i];
          //}
          //s += " are fixed";
        }
      }
    return s;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 
