import java.util.Vector;

public class NLSubjectCombiner extends NLSubjectIsolator {
  
  //Important inherited fields:
  //NLPhraseEvaluator[] evaluators;
  
  public NLSubjectCombiner(String phrase) {
    super(phrase);
    isolate();
  }
  
  public String eval(String[][] vars) {
    String s = "";
    if (checkSuitability(vars)) {
      //Build the initial part.
      s += evaluators[0].eval(vars[0]);
      //Now the subject.
      String[] a = new String[vars.length];
      for (int i = 0; i < a.length; i++) {
        a[i] = evaluators[1].eval(vars[i]);
      }
      if (a.length == 1) {
        s += a[0];
      } else if (a.length == 2) {
        s += a[0] + " and " + a[1];
      } else {
        for (int i = 0; i < a.length; i++) {
          s += (i>0?", ":"")+(i==a.length-1?"and ":"")+a[i];
        }
      }
      //And finally the last part.
      s += evaluators[2].eval(vars[0]);
    }
    return s;
  }
  
  public boolean checkSuitability(String[][] vars) {
  //The passed variable arrays are suitable just in case they agree
  //everywhere outside the subject.
    boolean okay = true;
    //First we check what comes before the subject.
    Evaluable[] a = evaluators[0].getPartsArray();
    if (a!=null) {
      okay &= checkPart(a,vars);
    }
    //And now what comes after.
    a = evaluators[2].getPartsArray();
    if (a!=null) {
      okay &= checkPart(a,vars);
    }
    return okay;
  }

  private boolean checkPart(Evaluable[] a, String[][] vars) {
    Evaluable ev; boolean okay = true;
    for (int i = 0; i < a.length; i++) {
      ev = a[i];
      if (
        ev.type.equals(Evaluable.VAR_TYPE) ||
        ev.type.equals(Evaluable.STRIP_VAR_TYPE)
      ) {
        int n = ev.varNum;
        okay &= checkForConstantColumn(vars,n);
      }
    }
    return okay;
  }
 
  public boolean checkForConstantColumn(String[][] aa, int j) {
    if (aa==null || aa.length==0) {return false;}
    for (int i = 0; i < aa.length; i++) {
      if (aa[i]==null || aa[i].length < j+1) {return false;}
    }
    String c = aa[0][j];
    for (int i = 1; i < aa.length; i++) {
      if (!aa[i][j].equals(c)) {return false;}
    }
    return true;
  }

}
